#!/usr/bin/env python

# The Python version of qwt-0.3/examples/bode/bode.cpp

# To get an impression of the expressive power of Numeric,
# compare the Python and C++ versions of recalc()

import glob, os, sys

# The next statements assure importation of qwt from the
# local source tree. This is useful for debugging and testing.
globs = glob.glob('../build/lib.*/qwt')
if globs:                               # found PyQwt modules made by distutils
    sys.path.insert(0, os.path.split(globs[0])[0])

from qt import *
from qwt import *
from Numeric import *

zoomInfo = 'Zoom: Press mouse button and drag'
cursorInfo = 'Cursor Pos: Press mouse button in plot region'

class InputFrame(QFrame):

    def __init__(self, *args):
        apply(QFrame.__init__, (self,) + args)

        lblDamp = QLabel('Damping Factor', self)
        lblDamp.setGeometry(20, 20, 100, 20);

        self.cntDamp = QwtCounter(self)
        self.cntDamp.setGeometry(120, 20, 120, 20)
        self.cntDamp.setRange(0.01, 5.0, 0.01)
        self.cntDamp.setValue(0.01)

        self.lblInfo = QLabel(cursorInfo, self)
        self.lblInfo.setFont(QFont('Helvetica', 8))
        self.lblInfo.setGeometry(20, 5, 230, 10)

        self.btnZoom = QPushButton('Zoom', self)
        self.btnZoom.setGeometry(300, 15, 100, 25)

        self.btnPrint = QPushButton('Print', self)
        self.btnPrint.setGeometry(420, 15, 100, 25)

class BodeDemo(QWidget):

    def __init__(self, *args):
        apply(QWidget.__init__, (self,) + args)

        self.fZoomActive = self.fZoom = 0;

        self.plot = plot =  QwtPlot(self)
        self.input = input = InputFrame(self)

        input.setFrameStyle(QFrame.Panel | QFrame.Raised)
        input.setLineWidth(2)

        plot.setTitle('Frequency Response of a Second-Order System');
    
        self.curve1 = plot.insertCurve('Amplitude')
        self.curve2 = plot.insertCurve('Phase')

        plot.setCurvePen(self.curve1, QPen(Qt.yellow))
        plot.setCurvePen(self.curve2, QPen(Qt.cyan))

        plot.enableGridXMin()
        plot.enableAxis(QwtPlot.yRight)
        plot.setGridMajPen(QPen(Qt.white, 0, Qt.DotLine))
        plot.setGridMinPen(QPen(Qt.gray, 0, Qt.DotLine))

        plot.setAxisTitle(QwtPlot.xBottom, 'Normalized Frequency')
        plot.setAxisTitle(QwtPlot.yLeft, 'Amplitude [dB]')
        plot.setAxisTitle(QwtPlot.yRight, 'Phase [deg]')
        
        plot.setAxisOptions(QwtPlot.xBottom, QwtAutoScale.Logarithmic)
        plot.setAxisMaxMajor(QwtPlot.xBottom, 6)
        plot.setAxisMaxMinor(QwtPlot.xBottom, 10)
        
        plot.setCurveYAxis(self.curve1, QwtPlot.yLeft)
        plot.setCurveYAxis(self.curve2, QwtPlot.yRight)
    
        self.mrk1 = plot.insertMarker()
        plot.setMarkerLineStyle(self.mrk1, QwtMarker.VLine)
        plot.setMarkerPos(self.mrk1, 0.0, 0.0)
        plot.setMarkerLabelAlign(self.mrk1, Qt.AlignRight | Qt.AlignBottom)
        plot.setMarkerPen(self.mrk1, QPen(Qt.green, 0, Qt.DashDotLine))
        plot.setMarkerFont(self.mrk1, QFont('Helvetica', 10, QFont.Bold))

        self.mrk2 = plot.insertLineMarker('', QwtPlot.yLeft)
        plot.setMarkerLabelAlign(self.mrk2, Qt.AlignRight | Qt.AlignBottom)
        plot.setMarkerPen(self.mrk2, QPen(
            QColor(200, 150, 0), 0, Qt.DashDotLine))
        plot.setMarkerFont(self.mrk2, QFont('Helvetica', 10, QFont.Bold))
        plot.setMarkerSymbol(self.mrk2, QwtSymbol(
            QwtSymbol.Diamond, QBrush(Qt.yellow), QPen(Qt.green), QSize(7,7)))
            
        plot.setPlotBackground(Qt.darkBlue)
        plot.enableLegend(1)
        plot.setLegendPos(Qwt.Bottom)
        plot.setLegendFrameStyle(QFrame.Box|QFrame.Sunken)

        # Numerical Python
        self.frequency = exp(log(10.0)*arrayrange(-2, 2.02, 0.04))
        self.damping = 0.01 # out of range if self.damping = 0.0

        self.recalc()
        QObject.connect(input.cntDamp, SIGNAL('valueChanged(double)'),
                        self.setDamp)
        QObject.connect(input.btnPrint, SIGNAL('clicked()'),
                        self.printPlot)
        QObject.connect(input.btnZoom, SIGNAL('clicked()'),
                        self.zoom)
        QObject.connect(plot, SIGNAL('plotMouseMoved(const QMouseEvent&)'),
                        self.plotMouseMoved)
        QObject.connect(plot, SIGNAL('plotMousePressed(const QMouseEvent&)'),
                        self.plotMousePressed)
        QObject.connect(plot, SIGNAL('plotMouseReleased(const QMouseEvent&)'),
                        self.plotMouseReleased)
        plot.enableOutline(1)
        plot.setOutlinePen(QPen(Qt.green))

    def printPlot(self):
        p = QPrinter()
        if p.setup():
            self.plot.printPlot(p, QwtFltrDim(200));

    def zoom(self):
        if self.fZoomActive:
            self.plot.setAxisAutoScale(QwtPlot.yLeft)
            self.plot.setAxisAutoScale(QwtPlot.yRight)
            self.plot.setAxisAutoScale(QwtPlot.xBottom)
            self.plot.replot()
            self.fZoom = 0
            self.fZoomActive = 0
        else:
            self.fZoom = not self.fZoom;
        if self.fZoom:
            self.input.btnZoom.setText('Unzoom')
            self.input.lblInfo.setText(zoomInfo)
        else:
            self.input.btnZoom.setText('Zoom')
            self.input.lblInfo.setText(cursorInfo)

    def setDamp(self, d):
        self.damping = d
        self.recalc()
        self.plot.replot()

    def plotMouseMoved(self, e):
        frequency = self.plot.invTransform(QwtPlot.xBottom, e.pos().x())
        amplitude = self.plot.invTransform(QwtPlot.yLeft, e.pos().y())
        phase = self.plot.invTransform(QwtPlot.yRight, e.pos().y())
        self.input.lblInfo.setText('Freq=%g, Ampl=%g, Phase=%g' %
                                   (frequency, amplitude, phase))
        
    def plotMousePressed(self, e):
        # Python semantics: self.pos = e.pos() does not work; force a copy
        self.xpos = e.pos().x()
        self.ypos = e.pos().y()
        self.plotMouseMoved(e)  # fake a mouse move to show the cursor position
        if (self.fZoom and not self.fZoomActive):
            self.plot.setOutlineStyle(Qwt.Rect) 
        else:
            self.plot.setOutlineStyle(Qwt.Cross)

    def plotMouseReleased(self, e):
        if (self.fZoom and not self.fZoomActive):
            plot = self.plot
            self.fZoomActive = 1;
            x1 = min(self.xpos, e.pos().x());
            x2 = max(self.xpos, e.pos().x());
            y1 = min(self.ypos, e.pos().y());
            y2 = max(self.ypos, e.pos().y());
            lim = 5 - (y2 - y1 + 1) / 2
            if lim > 0:
                y1 -= lim
                y2 += lim
            lim = 5 - (x2 - x1 + 1) / 2
            if lim > 0:
                x1 -= lim
                x2 += lim
            plot.setAxisScale(QwtPlot.yLeft,
                              plot.invTransform(QwtPlot.yLeft, y1),
                              plot.invTransform(QwtPlot.yLeft, y2))
            plot.setAxisScale(QwtPlot.yRight,
                              plot.invTransform(QwtPlot.yRight, y1),
                              plot.invTransform(QwtPlot.yRight, y2))
            plot.setAxisScale(QwtPlot.xBottom,
                              plot.invTransform(QwtPlot.xBottom, x1),
                              plot.invTransform(QwtPlot.xBottom, x2))
            plot.replot()
        self.input.lblInfo.setText(cursorInfo);
        self.plot.setOutlineStyle(Qwt.Triangle);

    def recalc(self):
        # Numerical Python: f, g, a and p are arrays!
        f = self.frequency
        g = 1.0/(1.0-f*f+2j*self.damping*f)
        a = 20.0*log10(abs(g))
        p = 180*arctan2(g.imag, g.real)/pi
        self.plot.setCurveData(self.curve1, f, a)
        self.plot.setCurveData(self.curve2, f, p)
        i3 = argmax(where(less(a, -3.0), a, -100.0))
        f3 = f[i3] - (a[i3]+3.0)*(f[i3]-f[i3-1])/(a[i3]-a[i3-1])
        self.plot.setMarkerPos(self.mrk1, f3, 0.0)
        self.plot.setMarkerLabel(self.mrk1, '-3 dB at f %4g' % f3)
        imax = argmax(a)
        self.plot.setMarkerPos(self.mrk2, f[imax], a[imax]);
        self.plot.setMarkerLabel(self.mrk2, 'Peak: %4g dB' % a[imax])
    
    def resizeEvent(self, e):
        r = QRect(0, 0, e.size().width(), e.size().height() - 50)
        self.plot.setGeometry(r)
        self.input.setGeometry(0, r.bottom() + 1, r.width(), 50)

# Admire! 
app = QApplication(sys.argv)
demo = BodeDemo()
app.setMainWidget(demo)
demo.resize(540, 400)
demo.show()
app.exec_loop()
