#!/usr/bin/env python

import glob, os, sys

# The next statements assure importation of qwt from the
# local source tree. This is useful for debugging and testing.
globs = glob.glob('../build/lib.*/qwt')
if globs:                               # found PyQwt modules made by distutils
    sys.path.insert(0, os.path.split(globs[0])[0])

from qt import *
from qwt import *
from Numeric import *
from LinearAlgebra import *
    
inductor = 84e-9
resistor = 0.7
capaP = 2.235e-12
capaS = 0.33e-12

def argument(z):
    return arctan(z.imaginary/z.real)

def norm(x, y):
    return sqrt(x*x+y*y)

def g_c(value, omega):
    return 1j*omega*value

def g_l(value, omega):
    return 1.0/(1j*omega*value)

def g_r(value, omega):
    return 1.0/value

# input node = -1
# ground node = -2

def zMatrix(circuit, omega):
    # find the number of modes and the node connected to the "input" branch
    highs = []
    for func, lo, hi, value in circuit:
        highs.append(hi)
        if lo == -1:
            inNode = hi
    nodes = 1 + max(highs)
    # the hard work
    z = zeros(len(omega), Complex)
    for i in range(len(omega)):
        a = zeros((nodes, nodes), Complex)  # nodal conductance matrix
        b = zeros(nodes, Complex)           # source current vector
        for func, lo, hi, value in circuit:
            g = func(value, omega[i])
            if lo == -2:                    # ground
                a[hi][hi] += g
            elif lo == -1:                  # input
                b[hi] += g
                a[hi][hi] += g
            else:
                a[lo][lo] += g
                a[hi][hi] += g
                a[lo][hi] -= g
                a[hi][lo] -= g
        v = solve_linear_equations(a, b)
        z[i] = 1.0/(b[0]*(1-v[0]))
    return z

def zPython(l, r, cp, cs, omega):
    return 1/(1j*omega*cp+1/(r+1j*omega*l))+1/(1j*omega*cs)

class SmithChart(QWidget):
    def __init__(self,*args):
        apply(QWidget.__init__,(self,) + args)
	
	# The QwtPlot widget
	self.gPlot = QwtPlot("Smith Chart", self)
        self.zPlot = QwtPlot("Impedance", self)
        # FIXME : look in .sip for the last option (step): maybe double = 0.0
        self.gPlot.setAxisScale(QwtPlot.xBottom, -1.0, 1.0)
        self.gPlot.setAxisScale(QwtPlot.yLeft, -1.0, 1.0)
	self.zPlot.setAxisScale(QwtPlot.xBottom, 300, 400)
	
        # circuit description
        Circuit4 = [
            (g_c, -1, 0, capaS),
            (g_r,  0, 1, resistor),
            (g_l, -2, 1, inductor),
            (g_c, -2, 0, capaP),
            ]

        # calculate impedance and reflection coefficient
        fs = 2*pi*arrayrange(130e6, 770e6, 0.25e6)
        zs = zMatrix(Circuit4, fs)
        ys = zPython(inductor, resistor, capaP, capaS, fs)
        gs = (zs-50.0)/(zs+50)
        fs = fs/(2e6*pi)

        gamma = self.gPlot.insertCurve("im(Gamma) vs re(Gamma)")
        self.gPlot.setCurveData(gamma, gs.real, gs.imag)
        curveReLadder = self.zPlot.insertCurve("re(zLadder)")
        curveImLadder = self.zPlot.insertCurve("im(zLadder)")
        curveRePython = self.zPlot.insertCurve("re(zPython)")
        curveImPython = self.zPlot.insertCurve("im(zPython)")
        self.zPlot.setCurveData(curveReLadder, fs, zs.real)
        self.zPlot.setCurvePen(curveReLadder, QPen(Qt.red))
        self.zPlot.setCurveData(curveImLadder, fs, zs.imag)
        self.zPlot.setCurvePen(curveImLadder, QPen(Qt.red))
        self.zPlot.setCurveData(curveRePython, fs, ys.real)
        self.zPlot.setCurvePen(curveRePython, QPen(Qt.blue))
        self.zPlot.setCurveData(curveImPython, fs, ys.imag)
        self.zPlot.setCurvePen(curveImPython, QPen(Qt.blue))


    def resizeEvent(self, e):
        x = e.size().width()/2
        y = e.size().height()
    	self.gPlot.resize(x, y)
	self.gPlot.move(0, 0)
        self.zPlot.resize(x, y)
        self.zPlot.move(x, 0)
	

if __name__ == '__main__':
    app = QApplication(sys.argv)
    chart = SmithChart()
    app.setMainWidget(chart)
    chart.resize(800, 400)
    chart.show()
    app.exec_loop()






