/*-*- c++ -*-******************************************************************
 * Qwt Widget Library 
 * Copyright (C) 1997   Josef Wilgen
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

#include "../include/qwt_curve.h"
#include "../include/qwt_math.h"
#include <qpntarry.h>
#include <qpainter.h>


/*!
  \brief Initialize data members
*/
void QwtCurve::init(const char *title)
{

    d_pen = QPen(QColor(255,255,255), 1);
    d_ref = 0.0;
    d_splineSize = 250;
    d_options = Auto;
    d_xraw = FALSE;
    d_yraw = FALSE;
    d_eraw = FALSE;
    hasErrors = FALSE;   
    
    if (title)
       d_title = title;
    else
       d_title = "";
	
    d_style = Lines;

}

/*!
  \brief Copy the contents of a curve into another curve
  \param c Todo...
*/
void QwtCurve::copy(const QwtCurve &c)
{

    d_ref = c.d_ref;
    d_xMap = c.d_xMap;
    d_yMap = c.d_yMap;
    d_sym = c.d_sym;
    d_pen = c.d_pen;
    d_title = c.d_title;
    d_xraw = FALSE;
    d_yraw = FALSE;
    d_style = c.d_style;
    d_eraw = FALSE;
    hasErrors = c.hasErrors;

    d_splineSize = c.d_splineSize;
    d_options = c.d_options;
    
    if (c.d_xraw)
       d_x.duplicate(c.d_x);
    else
       d_x = c.d_x;

    if (hasErrors) {
      if (c.d_eraw)
        d_e.duplicate(c.d_e);
      else
        d_e = c.d_e;
    }

    if (c.d_yraw)
       d_y.duplicate(c.d_y);
    else
       d_y = c.d_y;
}

//! Dtor
QwtCurve::~QwtCurve()
{
    if (d_xraw) d_x.resetRawData(d_x.data(), d_x.size());
    if (d_yraw) d_y.resetRawData(d_y.data(), d_y.size());
    if (d_eraw) d_e.resetRawData(d_e.data(), d_e.size());
}

/*!
  \brief Copy Constructor
  \warning If the c has attached its data in raw mode
  (see @QwtCurve::setRawData@), the copy constructor
  creates a deep copy of the data. The data are implicitly
  shared otherwise.
*/
QwtCurve::QwtCurve(const QwtCurve &c)
{
    init(c.d_title);
    copy(c);
}

/*!
  \brief Copy Assignment
  \warning If the c has attached its data in raw mode, the assignment operator
  creates a deep copy of the data. The data are implicitly
  shared otherwise.
  \sa setRawData()
*/
const QwtCurve& QwtCurve::operator=(const QwtCurve &c)
{
    copy(c);
    curveChanged();
    return *this;
}

/*!
  \brief Set the curve's drawing style

  Valid styles are:
  <dl>
  <dt>QwtCurve::NoCurve</dt>
  <dd>Don't draw a curve. Note: This doesn't affect the symbol. </dd>
  <dt>QwtCurve::Lines</dt>
  <dd>Connect the points with straight lines</dd>
  <dt>QwtCurve::Sticks</dt>
  <dd>Draw vertical sticks from a baseline which is defined by setBaseline().</dd>
  <dt>QwtCurve::Steps</dt>
  <dd>Connect the points with a step function. The step function
      is drawn from the left to the right or vice versa,
      depending on the 'Inverted' option.</dd>
  <dt>QwtCurves::Dots</dt>
  <dd>Draw dots at the locations of the data points. Note:
      This is different from a dotted line (see setPen()).</dd>
  <dt>QwtCurve::Spline</dt>
  <dd>Interpolate the points with a spline. The spline
      type can be specified with setOptions(),
      the size of the spline (= number of interpolated points)
      can be specified with setSplineSize().</dd></dl>
*/
void QwtCurve::setStyle(CurveStyle cs, int options)
{
    d_options = options;
    d_style = cs;
    curveChanged();
}

/*!
  \brief Assign a symbol
  \param s symbol
  \sa QwtSymbol
*/
void QwtCurve::setSymbol(const QwtSymbol &s )
{
    d_sym = s;
    curveChanged();
}


/*!
  \brief Assign a pen
  \param p New pen
*/
void QwtCurve::setPen(const QPen &p)
{
    d_pen = p;
    curveChanged();
}



/*!
  \brief Copy x-y data from specified arrays
  \param x pointer to x data
  \param y pointer to y data
  \param size size of x and y
*/
void QwtCurve::setData(double *x, double *y, int size)
{
    if (d_xraw) d_x.resetRawData(d_x.data(), d_x.size());
    if (d_yraw) d_y.resetRawData(d_y.data(), d_y.size());
    d_x.duplicate(x, size);
    d_y.duplicate(y, size);
    d_xraw = FALSE;
    d_yraw = FALSE;
    hasErrors = FALSE;
    curveChanged();
}


/*!
  \brief Copy x-y data and error data from specified arrays
  \param x pointer to x data
  \param y pointer to y data
  \param e pointer to error data
  \param data size
*/
void QwtCurve::setData(double *x, double *y, double *e, int size)
{
  if (d_xraw) d_x.resetRawData(d_x.data(), d_x.size());
  if (d_yraw) d_y.resetRawData(d_y.data(), d_y.size());
  if (d_eraw) d_e.resetRawData(d_e.data(), d_e.size());
  d_x.duplicate(x, size);
  d_y.duplicate(y, size);
  d_e.duplicate(e, size);
  d_xraw = FALSE;
  d_yraw = FALSE;
  d_eraw = FALSE;
  hasErrors = TRUE;
    curveChanged();
}

/*!
  \brief Attach raw data

  setRawData is provided for efficiency. In contrast to setData,
  it does not copy the data, so it is important to keep the pointers
  valid while they are attached. 

  The QwtCurve destructor does not delete the attached data, so you
  can safely call setRawData and setData several times subsequently.

  \param x pointer to x data
  \param y pointer to y data
  \param size size of x and y
*/
void QwtCurve::setRawData(double *x, double *y, int size)
{
    if (d_xraw) d_x.resetRawData(d_x.data(), d_x.size());
    if (d_yraw) d_y.resetRawData(d_y.data(), d_y.size());
    d_x.setRawData(x, size);
    d_y.setRawData(y, size);
    d_xraw = TRUE;
    d_yraw = TRUE;
    hasErrors = FALSE;
    curveChanged();
}
 

/*!
  \brief Attach arrays of x, y, and error data
  \param x pointer to x data
  \param y pointer to y data
  \param e pointer to error data
  \param data size
  \sa QwtCurve::setRawData(double *x, double *y, int size)
*/
void QwtCurve::setRawData(double *x, double *y, double *e, int size)
{
    if (d_xraw) d_x.resetRawData(d_x.data(), d_x.size());
    if (d_yraw) d_y.resetRawData(d_y.data(), d_y.size());
    if (d_eraw) d_e.resetRawData(d_e.data(), d_e.size());
    d_x.setRawData(x, size);
    d_y.setRawData(y, size);
    d_e.setRawData(e, size);
    d_xraw = TRUE;
    d_yraw = TRUE;
    d_eraw = TRUE;
    hasErrors = TRUE;
    curveChanged();
}

/*!
  \brief Assign a title to a curve
  \param title new title
*/
void QwtCurve::setTitle(const char *title)
{
    d_title = title;
    curveChanged();
}

/*!
  \brief find the smallest x value
*/
double QwtCurve::minXValue() const
{
    return qwtGetMin(d_x.data(),d_x.size());
}

/*!
  \brief find the smallest y value
*/
double QwtCurve::minYValue() const
{
    return qwtGetMin(d_y.data(), d_y.size());
}

/*!
  \brief find the largest x value
*/
double QwtCurve::maxXValue() const
{
    return qwtGetMax(d_x.data(), d_x.size());
}

/*!
  \brief find the largest y value
*/
double QwtCurve::maxYValue() const
{
    return qwtGetMax(d_y.data(), d_y.size());
}


/*!
  \brief Ctor
  \param title title of the curve	
*/
QwtCurve::QwtCurve(const char *title)
{
    init (title);
}

/*!
  \brief Checks if a range of indices is valid and corrects it if necessary
  \param i1 Index 1
  \param i2 Index 2
*/
int QwtCurve::verifyRange(int &i1, int &i2)
{
    int size = dataSize();

    if (size < 1) return 0;
    
    i1 = qwtLim(i1, 0, size-1);
    i2 = qwtLim(i2, 0, size-1);
    qwtSort(i1, i2, i1, i2);
    
    return (i2 - i1 + 1);
}

/*!
  \brief Assign maps and draw the curve
  \param p Painter
  \param xMap x map
  \param yMap y map
  \warning This function changes the maps and overrides any previous settings
  done by setRect(), setRange(), setMap(const QRect &r, double x1, double x2, bool xlog, double y1, double y2, bool ylog),
  and setMap(const QwtDiMap &mx, const QwtDiMap& my).
*/
void QwtCurve::draw(QPainter *p, const QwtDiMap &xMap, const QwtDiMap &yMap)
{

    d_xMap = xMap;
    d_yMap = yMap;
    draw(p);
    
}

/*!
  \brief Draw the curve
  \param p Painter
  \warning Before drawing, you have to assign a map using QwtCurve::setMap.
*/
void QwtCurve::draw(QPainter *p)
{

    if ( dataSize() <= 0 ) return;

    if (hasErrors) drawErrors(*p);
    
    switch (d_style)
    {
    case NoCurve:
	break;
    case Lines:
	drawLines(*p);
	break;
    case Sticks:
	drawSticks(*p);
	break;
    case Steps:
	drawSteps(*p);
	break;
    case Dots:
	drawDots(*p);
	break;
    case Spline:
	drawSpline(*p);
	break;
    default:
	drawDots(*p);
    }
    if (d_sym.style() != QwtSymbol::None)
    {
	drawSymbols(*p);
    }
}

//------------------------------------------------------------
//
//.F	QwtCurve::drawIntv
//	Draw a specified part of the curve
//
//.u	Syntax
//.f	void QwtCurve::drawIntv(QPainter *p, int imin, int imax)
//
//.u	Parameters
//.p	QPainter *p	--	Painter
//	int imin, int imax -- Interval boundaries as indices of the
//			      data arrays. The subset to be drawn includes both
//			      boundaries. 
//
//.u	Bugs
//	Drawing subintervals does not work with QwtCurve::Spline.
//
//------------------------------------------------------------
/*!
  \brief Draw a specified part of the curve
  \param p Painter
  \param imin lower boundary of the interval
  \param imax upper boundary of the interval
  \warning Drawing subintervals does not work with QwtCurve::Spline.
*/
void QwtCurve::drawIntv(QPainter *p, int imin, int imax)
{

    if ( dataSize() <= 0 ) return;

    if (hasErrors) drawErrors(*p,imin,imax);
    
    switch (d_style)
    {
    case NoCurve:
	break;
    case Lines:
    case Spline:
	drawLines(*p, imin, imax);
	break;
    case Sticks:
	drawSticks(*p, imin, imax);
	break;
    case Steps:
	drawSteps(*p, imin, imax);
	break;
    case Dots:
	drawDots(*p, imin, imax);
	break;
    default:
	drawDots(*p, imin, imax);
    }
    if (d_sym.style() != QwtSymbol::None)
    {
	drawSymbols(*p, imin, imax);
    }
    
}


/*!
  \brief Assign a bounding rectangle and draw the curve
  \param p Painter
  \param r bounding rectangle
  \warning This function changes the maps and overrides previous settings
  done by QwtCurve::setRect and QwtCurve::setMap.
*/
void QwtCurve::draw(QPainter *p, const QRect &r)
{
    setRect(r);
    draw(p);
}

/*!
  \brief draw lines
*/
void QwtCurve::drawLines(QPainter &p, int i1, int i2)
{
	int i, ip;
	int u1,v1,u2,v2;

	int size; 

	if (i2 < 0) i2 = dataSize() - 1; 
	size = verifyRange(i1, i2);

	if (size <= 1) return;
	
	if (d_pa.size() != uint(size))
	{
		if(!d_pa.resize(uint(size)))
		   return;
	}
	
	p.setPen(d_pen);

	transform(d_x[i1], d_y[i1], u1, v1);
	d_pa.setPoint(0,u1,v1);
	p.moveTo(u1,v1);
	for (i=i1+1,ip=1;i<=i2;i++,ip++)
	{
		transform(d_x[i], d_y[i], u2,v2);
		d_pa.setPoint(ip,u2,v2);
		u1 = u2;
		v1 = v2;
	}
	p.drawPolyline(d_pa);
	
}


/*!
  \brief Draw a spline
*/
void QwtCurve::drawSpline(QPainter &p)
{

    double xmin, xmax, ymin, ymax, delta;
    double *param;
    double dtmp;
    int i;
    int stype;
    int rc;
    int size = dataSize();
    double *txval = new double[size];
    double *tyval = new double[size];
	
	
    if ( (!txval) | (!tyval) | (!d_pa.resize(d_splineSize)) )
    {
	if (txval) delete[] txval;
	if (tyval) delete[] tyval;
	return;
    }

    //
    // Transform x and y values to window coordinates
    // to avoid a distinction between linear and
    // logarithmic scales.
    //
    for (i=0;i<size;i++)
    {
	txval[i] = d_xMap.xTransform(d_x[i]);
	tyval[i] = d_yMap.xTransform(d_y[i]);
    }

    if (! (d_options & (Yfx|Xfy|Parametric)))
    {
	
	if (qwtChkMono(txval, size))
	{
	    stype = Yfx;
	}
	else
	{
	    
	    if(qwtChkMono(tyval, size))
	    {
		stype = Xfy;
	    }
	    else
	    {
		stype = Parametric;
		if ( (d_options & Periodic) ||
		    ( (d_x[0] == d_x[size-1])
		    && (d_y[0] == d_y[size-1])))
		{
		    stype |= Periodic;  
		}
		
	    }
	}
    }
    else
    {
	stype = d_options;
    }
    
    if (stype & Parametric)
    {
	param = new double[size];
	if (param)
	{
	    //
	    // setup parameter vector
	    //
	    param[0] = 0.0;
	    for (i=1;i<size; i++)
	    {
		delta = sqrt( qwtSqr(txval[i] - txval[i-1]) + qwtSqr( tyval[i] - tyval[i-1]));
		param[i] = param[i-1] + qwtMax(delta, 1.0);
	    }
	    
	    //
	    // setup splines
	    rc = d_spx.recalc(param, txval, size, stype & Periodic);
	    if (!rc) rc = d_spy.recalc(param, tyval, size, stype & Periodic);
	    
	    if (rc)
	    {
		drawLines(p);
	    }
	    else
	    {
		
		// fill point array
		delta = param[size - 1] / double(d_splineSize-1);
		for (i=0;i<d_splineSize;i++)
		{
		    dtmp = delta * double(i);
		    d_pa.setPoint(i, int(floor (d_spx.value(dtmp) + 0.5)),
				  int(floor (d_spy.value(dtmp) + 0.5)));
		}
	    }
	    
	    
	    delete[] param;
	}
    }
    else if (stype & Xfy)
    {
	if (tyval[size-1] < tyval[0])
	{
	    qwtTwistArray(txval, size);
	    qwtTwistArray(tyval, size);
	}
	
	// 1. Calculate spline coefficients
	rc = d_spx.recalc(tyval, txval, size, stype & Periodic);
	
	if (rc)				// an error occurred
	{
	    drawLines(p);
	}
	else				// Spline OK
	{	
	    ymin = qwtGetMin(tyval, size);
	    ymax = qwtGetMax(tyval, size);
	    delta = (ymax - ymin) / double(d_splineSize - 1);
	    
	    for (i=0;i<d_splineSize;i++)
	    {
		dtmp = ymin + delta * double(i);
		d_pa.setPoint(i, int(floor(d_spx.value(dtmp) + 0.5)),
			      int(floor(dtmp + 0.5)));
	    }
	}
    }		
    else
    {
	if (txval[size-1] < txval[0])
	{
	    qwtTwistArray(tyval, size);
	    qwtTwistArray(txval, size);
	}
	
	
	// 1. Calculate spline coefficients
	rc = d_spy.recalc(txval, tyval, size, stype & Periodic);
	
	if (rc)				// error
	{
	    drawLines(p);
	}
	else				// Spline OK
	{
	    xmin = qwtGetMin(txval, size);
	    xmax = qwtGetMax(txval, size);
	    delta = (xmax - xmin) / double(d_splineSize - 1);
	    
	    for (i=0;i<d_splineSize;i++)
	    {
		dtmp = xmin + delta * double(i);
		d_pa.setPoint(i, int(floor (dtmp + 0.5)),
			      int(floor(d_spy.value(dtmp) + 0.5)));
	    }
	}
    }
    
    p.setPen(d_pen);
    p.drawPolyline(d_pa);
    
    delete[] txval;
    delete[] tyval;
    
}

/*!
  \brief Specify options for the drawing style	

  The options can be used to modify the drawing style.
  Options can be or-combined.
  The following options are defined:<dl>
  <dt>QwtCurve::Auto</dt>
  <dd>The default setting. For QwtCurve::spline,
      this means that the type of the spline is
      determined automatically, depending on the data.
      For all other styles, this means that y is
      regarded as a function of x.</dd>
  <dt>QwtCurve::Yfx</dt>
  <dd>Draws y as a function of x (the default). The
      baseline is interpreted as a horizontal line
      with y = baseline().</dd>
  <dt>QwtCurve::Xfy</dt>
  <dd>Draws x as a function of y. The baseline is
      interpreted as a vertical line with x = baseline().</dd>
  <dt>QwtCurve::Parametric</dt>
  <dd>For QwtCurve::Spline only. Draws a parametric spline.</dd>
  <dt>QwtCurve::Periodic</dt>
  <dd>For QwtCurve::Spline only. Draws a periodic spline.</dd>
  <dt>QwtCurve::Inverted</dt>
  <dd>For QwtCurve::Steps only. Draws a step function
      from the right to the left.</dd></dl>

  \param opt new options
*/
void QwtCurve::setOptions(int opt)
{
    d_options = opt;
    curveChanged();
}

/*!
  \brief Change the number of interpolated points
  \param s new size
  \warning The default is 250 points.
*/
void QwtCurve::setSplineSize(int s)
{
    d_splineSize = qwtMax(s, 10);
    curveChanged();
}


/*!
  \brief draw sticks
*/
void QwtCurve::drawSticks(QPainter &p, int i1, int i2)
{
    int i;
    int u,v;
    int u0, v0;
    int size;
    
    if (i2 < 0) i2 = dataSize() - 1; 
    size = verifyRange(i1, i2);

    if (size <= 0) return;
    
    p.setPen(d_pen);

    if (d_options & Xfy)
    {

	u0 = d_xMap.transform(qwtLim(d_ref, d_xMap.d1(), d_xMap.d2()));
    
	for (i=i1;i<=i2;i++)
	{
	    transform(d_x[i], d_y[i], u,v);
	    p.drawLine(u0, v, u,v);
	}
    }
    else
    {
	v0 = d_yMap.transform(qwtLim(d_ref, d_yMap.d1(), d_yMap.d2()));
    
	for (i=i1;i<=i2;i++)
	{
	    transform(d_x[i], d_y[i], u,v);
	    p.drawLine(u,v0,u,v);
	}
    }
    
}

/*!
  \brief Draw error bars
*/
void QwtCurve::drawErrors(QPainter &p, int i1, int i2)
{
    int i;
    int vl,vh,ul,uh;
    int u, v, w;
    int size;

    if (!hasErrors) return;

    if (i2 < 0) i2 = dataSize() - 1;
    size = verifyRange(i1, i2);

    if (size <= 0) return;

    p.setPen(d_pen);
    if (size < 1) return;

    w = (d_sym.size().width() - 1) / 2;

    if (d_options & Xfy)
    {

        // u0 = d_xMap.transform(qwtLim(d_ref, d_xMap.d1(), d_xMap.d2()));

        for (i=i1;i<=i2;i++)
        {
          transform(d_x[i]-d_e[i], d_y[i], ul, v);
            transform(d_x[i]+d_e[i], d_y[i], uh, v);
            p.drawLine(ul, v, uh, v);
          if (w) {
            p.drawLine(ul, v-w, ul, v+w);
            p.drawLine(uh, v-w, uh, v+w);
          }
        }
    }
    else
    {
        // v0 = d_yMap.transform(qwtLim(d_ref, d_yMap.d1(), d_yMap.d2()));

        for (i=i1;i<=i2;i++)
        {
          transform(d_x[i], d_y[i]-d_e[i], u, vl);
            transform(d_x[i], d_y[i]+d_e[i], u, vh);
            p.drawLine(u, vl, u, vh);
          if (w) {
            p.drawLine(u-w, vl, u+w, vl);
            p.drawLine(u-w, vh, u+w, vh);
          }
        }
    }

}


/*!
  \brief Draw dots
*/
void QwtCurve::drawDots(QPainter &p, int i1, int i2)
{
    int i;
    int u,v;
    int size;
    
    if (i2 < 0) i2 = dataSize() - 1; 
    size = verifyRange(i1, i2);

    if (size <= 0) return;
    
    p.setPen(d_pen);
    
    for (i=1;i<=i2;i++)
    {
	transform(d_x[i], d_y[i], u,v);
	p.drawPoint(u,v);
    }
    
}


/*!
  \brief Draw step function
*/
void QwtCurve::drawSteps(QPainter &p, int i1, int i2)
{
    int i,ip;
    int u1,v1,u2,v2,w1,w2;
    int size;
    
    if (i2 < 0) i2 = dataSize() - 1; 
    size = verifyRange(i1, i2);

    if (size <= 1) return;
    
    if (d_pa.size() != uint (2*size - 1))
    {
	if (!d_pa.resize(uint(2*size - 1)))
	   return;
    }
    
    p.setPen(d_pen);
    
    w1 = (d_pen.width() +1) /2;
    w2 = d_pen.width() / 2;
    
    transform(d_x[i1], d_y[i1], u1, v1);
    d_pa.setPoint(0, u1,v1);   

    if (((d_options & Xfy) && (!(d_options & Inverted)))
	|| ((d_options & Inverted) && (!(d_options & Xfy))))
    {
	for (i=i1+1, ip=2;i<=i2;i++, ip+=2)
	{
	    transform(d_x[i], d_y[i], u2,v2);
	    d_pa.setPoint(ip - 1, u1,v2);   
	    d_pa.setPoint(ip, u2,v2);
	    u1 = u2;
	    v1 = v2;
	}
    }
    else
    {
	for (i=i1+1,ip=2;i<= i2;i++,ip+=2)
	{
	    transform(d_x[i], d_y[i], u2,v2);
	    d_pa.setPoint(ip - 1, u2,v1);   
	    d_pa.setPoint(ip, u2,v2);
	    u1 = u2;
	    v1 = v2;
	}
    }
    
    p.drawPolyline(d_pa);
    
    
}

/*!
  \brief Transform
*/
void QwtCurve::transform(double x, double y, int &u, int &v)
{
    u = d_xMap.transform(x);
    v = d_yMap.transform(y);
}


/*!
  \brief Draw symbols
*/
void QwtCurve::drawSymbols(QPainter &p, int i1, int i2)
{
    int i, u, v;
    int size;
    
    if (i2 < 0) i2 = qwtMin(d_x.size(), d_y.size()) - 1; 
    size = verifyRange(i1, i2);

    if (size < 1) return;
    int w2 = (d_sym.size().width() - 1) / 2;
    int h2 = (d_sym.size().height() - 1) / 2;
    
    for (i=i1;i<=i2;i++)
    {
	transform(d_x[i], d_y[i], u,v);
	d_sym.draw(&p, u - w2, v - h2);
    }
}


/*!
  \brief Specify x and y ranges
  \param r bounding rectangle
  \param x1 left boundary of the x axis
  \param x2 right boundary of the y axis
  \param xlog logarithmic x division if nonzero
  \param y1 lower boundary of the y axis
  \param y2 upper boundary of the y axis
  \param ylog logarithmic y division if nonzero
  \warning This function overrides the settings of setRect() and setRange().
*/
void QwtCurve::setMap(const QRect &r, double x1, double x2, bool xlog, double y1, double y2, bool ylog)
{
    d_xMap.setDblRange(x1, x2, xlog);
    d_yMap.setDblRange(y1, y2, ylog);
    d_xMap.setIntRange(r.left(), r.right());
    d_yMap.setIntRange(r.top(), r.bottom());
}

/*!
  \brief Changes the drawing region of the map
  \param r  Bounding rectangle
  \warning This function is intended to be used in combination
  with setRange(). It modifies the maps and conflicts with setMap()
*/
void QwtCurve::setRect(const QRect &r)
{
    d_xMap.setIntRange(r.left(), r.right());
    d_yMap.setIntRange(r.top(), r.bottom());
}


/*!
  \brief Changes the data range of the map
  \param x1 lower boundary of the x range
  \param x2 upper boundary of the x range
  \param xlog TRUE if x range is logarithmic
  \param y1 lower boundary of the x range
  \param y2 upper boundary of the y range
  \param ylog TRUE if y range is logarithmic
  \warning This function is intended to be used in combination
  with setRect(). It modifies the maps and conflicts with
  setMap().
*/
void QwtCurve::setRange(double x1, double x2, bool xlog, double y1, double y2, bool ylog)
{
    d_xMap.setDblRange(x1, x2, xlog);
    d_yMap.setDblRange(y1, y2, ylog);
}

/*!
  \brief Assign the maps for the x and y axes
  \param xmap x map
  \param ymap y map
  \warning This function overrides the settings of setRect() and setRange().
*/
void QwtCurve::setMap(const QwtDiMap &xmap, const QwtDiMap &ymap)
{
    d_xMap = xmap;
    d_yMap = ymap;
}

/*!
  \brief Set the value of the baseline

  The baseline is needed for the QwtCurve::Sticks
  drawing style. The default value is 0.0. The interpretation
  of the baseline depends on the style options. With QwtCurve::Yfx,
  the baseline is interpreted as a horizontal line at y = baseline(),
  with QwtCurve::Yfy, it is interpreted as a vertical line at
  x = baseline().
  \param ref baseline
  \sa setStyle(), setOptions()
*/
void QwtCurve::setBaseline(double ref)
{
    d_ref = ref;
    curveChanged();
}


/*!
  \brief Return the size of the data arrays
*/
int QwtCurve::dataSize() const
{
    return qwtMin(d_x.size(), d_y.size());
}
