/*-*- c++ -*-******************************************************************
 * Qwt Widget Library 
 * Copyright (C) 1997   Josef Wilgen
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

#include "../include/qwt_grid.h"


//! ctor
QwtGrid::QwtGrid()
{
    d_xMinEnabled = FALSE;
    d_yMinEnabled = FALSE;
    d_xEnabled = TRUE;
    d_yEnabled = TRUE;
}

//! dtor
QwtGrid::~QwtGrid()
{
    
}

/*!
  \brief Enable or disable vertical gridlines
  \param tf Enable (TRUE) or disable

  \sa Minor gridlines can be enabled or disabled with
      enableXMin()
*/
void QwtGrid::enableX(bool tf)
{
    d_xEnabled = tf;
    gridChanged();
}

/*!
  \brief Enable or disable horizontal gridlines
  \param tf Enable (TRUE) or disable
  \sa Minor gridlines can be enabled or disabled with enableYMin()
*/
void QwtGrid::enableY(bool tf)
{
    d_yEnabled = tf;  
    gridChanged();
}

/*!
  \brief Enable or disable  minor vertical gridlines.
  \param tf Enable (TRUE) or disable
  \sa enableX()
*/
void QwtGrid::enableXMin(bool tf)
{
    d_xMinEnabled = tf;
    gridChanged();
}

/*!
  \brief Enable or disable minor horizontal gridlines
  \param tf Enable (TRUE) or disable
  \sa enableY()
*/
void QwtGrid::enableYMin(bool tf)
{
    d_yMinEnabled = tf;
    gridChanged();
}

/*!
  \brief Assign an x axis scale division
  \param sx Scale division
  \warning QwtGrid uses implicit sharing (see Qt Manual) for
  the scale divisions.
  \sa QwtScaleDiv
*/
void QwtGrid::setXDiv(const QwtScaleDiv &sx)
{
    d_sdx = sx;
    gridChanged();
}

/*!
  \brief Assign a y axis division
  \param sy Scale division
  \warning QwtGrid uses implicit sharing (see Qt Manual) for
  the scale divisions.
  \sa QwtScaleDiv
*/
void QwtGrid::setYDiv(const QwtScaleDiv &sy)
{
    d_sdy = sy;    
    gridChanged();
}

/*!
  \brief Assign a pen for both major and minor gridlines
  \param p Pen
*/
void QwtGrid::setPen(const QPen &p)
{
    d_majPen = p;
    d_minPen = p;
    gridChanged();
}

/*!
  \brief Assign a pen for the major gridlines
  \param p Pen
*/
void QwtGrid::setMajPen(const QPen &p)
{
    d_majPen = p;
    gridChanged();
}

/*!
  \brief Assign a pen for the minor gridlines
  \param p Pen
*/
void QwtGrid::setMinPen(const QPen &p)
{
    d_minPen = p;  
    gridChanged();
}

/*!
  \brief Draw the grid
  
  The grid is drawn into the bounding rectangle such that 
  gridlines begin and end at the rectangle's borders. The X and Y
  maps are used to map the scale divisions into the drawing region
  screen.
  \param p  Painter
  \param r Bounding rectangle
  \param mx X axis map
  \param my Y axis 
*/
void QwtGrid::draw(QPainter *p, const QRect &r, const QwtDiMap &mx, const QwtDiMap &my)
{

    int i, x = 0, y = 0;
    int x1, x2, y1, y2;
    x1 = r.left();
    x2 = r.right();
    y1 = r.top();
    y2 = r.bottom();

    //
    //  draw minor gridlines
    //
    p->setPen(d_minPen);
    
    if (d_xEnabled && d_xMinEnabled)
    {
	for (i=0;i<d_sdx.minCnt();i++)
	{
	    x = mx.transform(d_sdx.minMark(i));
	    if ((x >= x1) && (x <= x2))
	       p->drawLine(x, y1, x, y2);
	}
	
    }

    if (d_yEnabled && d_yMinEnabled)
    {
	for (i=0;i<d_sdy.minCnt();i++)
	{
	    y = my.transform(d_sdy.minMark(i));
	    if ((y >= y1) && (y <= y2))
	       p->drawLine(x1, y, x2, y);
	}
	
    }

    //
    //  draw major gridlines
    //
    p->setPen(d_majPen);
    
    if (d_xEnabled)
    {
	for (i=0;i<d_sdx.majCnt();i++)
	{
	    x = mx.transform(d_sdx.majMark(i));
	    if ((x >= x1) && (x <= x2))
	       p->drawLine(x, y1, x, y2);
	}
	
    }
    
    if (d_yEnabled)
    {
	for (i=0;i<d_sdy.majCnt();i++)
	{
	    y = my.transform(d_sdy.majMark(i));
	    if ((y >= y1) && (y <= y2))
	       p->drawLine(x1, y, x2, y);
	}
    }
    
}

/*!
  \brief Assignment operator 
  \param g Grid to be assigned
*/
const QwtGrid & QwtGrid::operator=(const QwtGrid &g)
{
    d_xEnabled = g.d_xEnabled;
    d_yEnabled = g.d_yEnabled;
    d_xMinEnabled = g.d_xMinEnabled;
    d_yMinEnabled = g.d_yMinEnabled;
    d_sdx = g.d_sdx;
    d_sdy = g.d_sdy;

    d_majPen = g.d_majPen;
    d_minPen = g.d_minPen;
    
    return *this;
}
