/*-*- c++ -*-******************************************************************
 * Qwt Widget Library 
 * Copyright (C) 1997   Josef Wilgen
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

#include "../include/qwt_knob.h"

#include <qpainter.h>
#include <qpalette.h>
#include "../include/qwt_math.h"

/*!
  \brief Constructor
  \param parent	Parent widget
  \param name Name 
*/
QwtKnob::QwtKnob(QWidget* parent, const char *name)
: QwtSliderBase(parent,name)
{
    d_borderWidth = 2;
    d_borderDist = 4;
    d_totalAngle = 270.0;
    d_scaleDist = 4;
    d_hasScale = 0;
    d_symbol = Line;
    d_maxScaleTicks = 11;
    d_knobWidth = 50;
   d_faceColor = backgroundColor();
    d_markerColor = foregroundColor();
    d_dotWidth = 8;
    
    setMinimumSize(30,30);
    setUpdateTime(50);
}

//! Destructor
QwtKnob::~QwtKnob() 
{
}

/*!
  \brief Set the total angle by which the knob can be turned
  \param angle Angle in degrees.

  The default angle is 270 degrees. It is possible to specify
  an angle of more than 360 degrees so that the knob can be
  turned several times around its axis.   
*/
void QwtKnob::setTotalAngle (double angle) 
{
    if (angle < 10.0)
       d_totalAngle = 10.0;
    else
       d_totalAngle = angle;
    d_scale.setAngleRange( -0.5 * d_totalAngle, 0.5 * d_totalAngle);
}


/*!
  \brief Draw the knob
  \param p painter
  \param r borders of the knob
*/
void QwtKnob::drawKnob(QPainter *p, const QRect &r)
{

    QRect aRect;

    QColorGroup g = colorGroup();
    QPen pn;
    int bw2 = d_borderWidth / 2;
    

    aRect.setRect( r.x() + bw2,
		  r.y() + bw2,
		  r.width() - 2*bw2,
		  r.height() - 2*bw2);
    
    //
    // draw button face
    //
    p->setPen(NoPen);
    p->setBrush(d_faceColor);
    p->drawEllipse(aRect);

    //
    // draw button shades
    //
    pn.setWidth(d_borderWidth);

    
    pn.setColor(g.light());
    p->setPen(pn);
    p->drawArc(aRect, 45*16,180*16);

    pn.setColor(g.dark());
    p->setPen(pn);
    p->drawArc(aRect, 225*16,180*16);

    //
    // draw marker
    //
    drawMarker(p, d_angle, d_markerColor);
    
}


/*!
  \brief Notify change of value
  
  Sets the slider's value to the nearest multiple
  of the step size.
*/
void QwtKnob::valueChange() 
{
    recalcAngle();
    d_newVal++;
    repaint(d_kRect, FALSE);
    QwtSliderBase::valueChange();
}

/*!
  \brief Determine the value corresponding to a specified position

  Called by QwtSliderBase
  \param p point
*/
double QwtKnob::getValue(const QPoint &p)
{

    double newValue;
    double oneTurn;
    double eqValue;
    double arc;
    
    const QRect& r = rect();

    double dx = double((r.x() + r.width() / 2) - p.x() );
    double dy = double((r.y() + r.height() / 2) - p.y() );

    arc = atan2(-dx,dy) * 180.0 / M_PI;

    newValue =  0.5 * (minValue() + maxValue())
       + (arc + d_nTurns * 360.0) * (maxValue() - minValue())
	  / d_totalAngle;
    
    oneTurn = fabs(maxValue() - minValue()) * 360.0 / d_totalAngle;
    eqValue = value() + d_mouseOffset;
    
    if (fabs(newValue - eqValue) > 0.5 * oneTurn) 
    {
	if (newValue < eqValue) 
	   newValue += oneTurn;
	else
	   newValue -= oneTurn;
    }

    return newValue;	

}



/*!
  \brief Set the scrolling mode and direction

  Called by QwtSliderBase
  \param p Point in question
*/
void QwtKnob::getScrollMode(const QPoint &p, int &scrollMode, int &direction)
{
    int dx, dy, r;
    double arc;

    r = d_kRect.width() / 2;

    dx = d_kRect.x() + r - p.x();
    dy = d_kRect.y() + r - p.y();
    
    if ( (dx * dx) + (dy * dy) <= (r * r)) // point is inside the knob
    {
	scrollMode = ScrMouse;
	direction = 0;
    }
    else								// point lies outside
    {
	scrollMode = ScrTimer;
	arc = atan2(double(-dx),double(dy)) * 180.0 / M_PI;
	if ( arc < d_angle)
	   direction = -1;
	else if (arc > d_angle)
	   direction = 1;
	else
	   direction = 0;
    }
    return;
}



/*!
  \brief Notify a change of the range

  Called by QwtSliderBase
*/
void QwtKnob::rangeChange()
{
    if (!hasUserScale())
    {
	d_scale.setScale(minValue(), maxValue(),
			 d_maxMajor, d_maxMinor);
    }
    recalcAngle();
    resize(size());
    repaint(FALSE);
}

/*!
  \brief Qt Resize Event
*/
void QwtKnob::resizeEvent(QResizeEvent *e)
{
    int width, width_2;

    const QRect &r = rect();

    width = qwtMin(qwtMin(r.height(), r.width()), d_knobWidth);
    width_2 = width / 2;
    
    d_kRect.setRect(r.x() + r.width() / 2 - width_2,
		    r.y() + r.height() / 2 - width_2,
		    width, width);

    d_scale.setGeometry( d_kRect.x() - d_scaleDist,
			d_kRect.y() - d_scaleDist,
			width + 2 * d_scaleDist, QwtScaleDraw::Round );
}

/*!
  \brief Repaint the knob
*/
void QwtKnob::paintEvent(QPaintEvent *e)
{
    QPainter p;
    const QRect &r = e->rect();

    if( p.begin(this))
    {
	
	if ( (r == d_kRect) && d_newVal ) // event from valueChange() 
	{
	    if (d_newVal > 1)		// lost paintEvents()?
	    {
		drawKnob(&p, d_kRect);
	    }
	    else
	    {
		drawMarker(&p, d_oldAngle, d_faceColor);
		drawMarker(&p, d_angle, d_markerColor);
	    }
	    
	}
	else
	{
	    p.fillRect(rect(), backgroundColor());
	    d_scale.draw(&p);
	    drawKnob(&p, d_kRect);
	}
	
    }
    
    p.end();
    
    d_newVal = 0;
    
}

/*!
  \brief Draw the marker at the knob's front
  \param p Painter
  \param arc Angle of the marker
  \param c Marker color
*/
void QwtKnob::drawMarker(QPainter *p, double arc, const QColor &c)
{
    
    QPen pn;
    int radius;
    double rb,re;
    double rarc;
    
    rarc = arc * M_PI / 180.0;
    double ca = cos(rarc);
    double sa = - sin(rarc);
    radius = d_kRect.width() / 2 - d_borderWidth;
    if (radius < 3) radius = 3;	
    int ym = d_kRect.y() + radius + d_borderWidth;
    int xm = d_kRect.x() + radius + d_borderWidth;
    
    switch (d_symbol)
    {
    case Dot:
	
	p->setBrush(c);
	p->setPen(NoPen);
	rb = double(qwtMax(radius - 4 - d_dotWidth / 2, 0));
	p->drawEllipse(xm - int(floor (sa * rb + 0.5)) - d_dotWidth / 2,
		       ym - int(floor (ca * rb + 0.5)) - d_dotWidth / 2,
		       d_dotWidth, d_dotWidth);
	
	break;
	
    case Line:
	
	pn.setColor(c);
	pn.setWidth(2);
	p->setPen(pn);
	
	rb = qwtMax(double((radius - 4) / 3.0), 0.0);
	re = qwtMax(double(radius - 4), 0.0);
	
	p->drawLine (xm - int (floor (sa * rb + 0.5)),
		     ym - int (floor (ca * rb + 0.5)),
		     xm - int (floor (sa * re + 0.5)),
		     ym - int (floor (ca * re + 0.5)));
	
	break;
    }
    
    
}

/*!
  \brief Change the knob's width.

  The specified width must be >= 5, or it will be clipped.
  \param w New width
*/
void QwtKnob::setKnobWidth(int w)
{
    d_knobWidth = qwtMax(w,5);
    resize(size());
    repaint(FALSE);
}

/*!
  \brief Set the knob's border width
  \param bw new border width
*/
void QwtKnob::setBorderWidth(int bw)
{
    d_borderWidth = qwtMax(bw, 0);
    resize(size());
    repaint(FALSE);
}

/*!
  \brief Recalculate the marker angle corresponding to the
	current value
*/
void QwtKnob::recalcAngle()
{
    d_oldAngle = d_angle;
    
    //
    // calculate the angle corresponding to the value
    //
    if (maxValue() == minValue())
    {
	d_angle = 0;
	d_nTurns = 0;
    }
    else 
    {
	d_angle = (value() - 0.5 * (minValue() + maxValue()))
	   / (maxValue() - minValue()) * d_totalAngle;
	d_nTurns = floor((d_angle + 180.0) / 360.0);
	d_angle = d_angle - d_nTurns * 360.0;
	
    }
    
}


/*!
  \todo Documentation
*/
void QwtKnob::scaleChange()
{
    repaint(FALSE);
}

/*!
  \todo Documentation
*/
void QwtKnob::fontChange(const QFont &oldFont)
{
    repaint(FALSE);
}









