/*-*- c++ -*-******************************************************************
 * Qwt Widget Library 
 * Copyright (C) 1997   Josef Wilgen
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

#include "../include/qwt_pixframe.h"
#include "../include/qwt_math.h"
#include <qpainter.h>

//! CTOR
QwtPixFrame::QwtPixFrame(QWidget *parent, const char *name)
    : QFrame(parent, name)
{
    d_bg = backgroundColor();
    d_adjustPixmap = TRUE;
}


//! DTOR
QwtPixFrame::~QwtPixFrame()
{
    
}


//! Paint event
void QwtPixFrame::paintEvent(QPaintEvent *e)
{
    QRect r = contentsRect();
    QRect ri;
    QPainter p;

    p.begin(this);

    // redraw frame if necessary
    if (!r.contains(e->rect(), TRUE))
       drawFrame(&p);
    
    // redraw pixmap portion
    if (r.intersects(e->rect()))
    {
	ri = r.intersect(e->rect());

	//
	// repaint pixmap portion
	//
	p.drawPixmap(ri.x(), ri.y(), d_pix,
		     ri.x() - r.x(), ri.y() - r.y(),
		     qwtMin(ri.width(), d_pix.width()),
		     qwtMin(ri.height(), d_pix.height()));

    }

    p.end();
    
	
}


/*!
  \brief Assign a pixmap

  If the variable size feature is enabled (disabled by default),
  the size of the frame will not change when a new pixmap
  has been assigned. 
 */
void QwtPixFrame::setPixmap(const QPixmap &pix)
{
    d_pix = pix;
    if (!d_adjustPixmap)
       adjustFrame();
}


//! Resize the pixmap according to the frame size. 
void QwtPixFrame::adjustPixmap()
{
    if (contentsRect().size() != d_pix.size())
    {
	d_pix.resize(contentsRect().size());
	clearPixmap();
    }
}


//! Resize the frame according to the pixmap size. 
void QwtPixFrame::adjustFrame()
{
    setFixedSize(d_pix.width() + 2 * frameWidth(), d_pix.height() + 2 * frameWidth());
}


/*!
  \brief Fill the pixmap with a specified color
  \param c Fill color
*/
void QwtPixFrame::fillPixmap(const QColor &c)
{
    d_pix.fill(c);
}


void QwtPixFrame::resizeEvent(QResizeEvent *e)
{

    QFrame::resizeEvent(e);
    if (d_adjustPixmap)
    {
	adjustPixmap();
	emit pixmapResized();
    }
}


//! Notify a change of the frame
void QwtPixFrame::frameChanged()
{
    if (d_adjustPixmap)
    {
	this->resize(this->size());
	this->repaint(FALSE);
    }
    else
       adjustFrame();
}


/*!
  \brief Maps the pixmap to the screen
  
  This function synchronizes the screen contents with the
  current contents of the pixmap. It generates a paint event
  which updates the widget.

  \warning When the widget gets resized, a paint event is generated
  automatically after the pixmapResized() signal has been
  emitted. It is not necessary to call updatePixmap()
  within a slot which is connected to pixmapResized().
*/
void QwtPixFrame::updatePixmap()
{
    repaint(contentsRect(), FALSE);
}


/*!
  \brief Repaint a part of the pixmap
  \param r part of the pixmap to be redrawn
           in pixmap coordinates
*/
void QwtPixFrame::updatePixmapRect(const QRect &r)
{
    QRect pr = d_pix.rect().intersect(r);
    if (pr.isValid() && (!pr.isNull()))
    {
	pr.moveTopLeft(pr.topLeft() + contentsRect().topLeft());
	repaint(pr, FALSE);
    }
}


/*!
  \brief Set resizing policy.
  \param tf Adjust pixmap size (TRUE) or frame size (FALSE).

  If tf is TRUE, the pixmap will be adjusted when the
  frame is resized or when the frame style is changed.
  If set to FALSE, the size of the pixmap will remain
  constant and the frame size will be adjusted.
  The default value is TRUE. 
*/
void QwtPixFrame::setVariableSize(bool tf)
{
    if (d_adjustPixmap != tf) 
    {
	d_adjustPixmap = tf;
	if (tf) 
	    adjustFrame();
	else
	   adjustPixmap();
    }
}


/*!
  \brief Assign a background color the the pixmap
  \param c New background color
*/
void QwtPixFrame::setPixmapBackground(const QColor &c)
{
    d_bg = c;
}


/*!
  \brief Clear the contents of the pixmap and fill it with
  its background color
*/
void QwtPixFrame::clearPixmap()
{
    d_pix.fill(d_bg);
}





