/*-*- c++ -*-******************************************************************
 * Qwt Widget Library 
 * Copyright (C) 1997   Josef Wilgen
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

#include "../include/qwt_plot.h"
#include "../include/qwt_plot_dict.h"
#include "../include/qwt_math.h"
#include "../include/qwt_legend.h"

//------------------------------------------------------------
//.C Accessing and Manipulating Curves
//------------------------------------------------------------

/*!
  Find the curve which is closest to a
  specified point in the plotting area.
  \param xpos
  \param ypos position in the plotting region
  \retval dist distance in points between (xpos, ypos) and the
               closest data point
  \return Key of the closest curve or 0 if no curve was found.
*/
long QwtPlot::closestCurve(int xpos, int ypos, int &dist) const
{
    double x,y;
    int index;
    return closestCurve(xpos, ypos, dist, x,y, index);
}

/*!
  Find the curve which is closest to a point in the plotting area.
  
  Determines the position and index of the closest data point.
  \param xpos
  \param ypos coordinates of a point in the plotting region
  \retval xval
  \retval yvalvalues of the closest point in the curve's data array
  \retval dist -- distance in points between (xpos, ypos) and the
                  closest data point
  \retval index -- index of the closest point in the curve's data array
  \return Key of the closest curve or 0 if no curve was found.
*/
long QwtPlot::closestCurve(int xpos, int ypos, int &dist, double &xval,
			   double &yval, int &index) const
{
    long rv = 0;
    QwtPlotCurve *c;
    QIntDictIterator<QwtPlotCurve> itc(*d_curves);

    double dmin = 1.0e10;
    double f, cx, cy;

    int i;
    
    itc.toFirst();
    while((c = itc.current()))
    {
	for (i=0; i<c->dataSize(); i++)
	{
	    cx = d_map[c->xAxis()].xTransform(c->x(i)) - double(xpos);
	    cy = d_map[c->yAxis()].xTransform(c->y(i)) - double(ypos);

	    f = qwtSqr(cx) + qwtSqr(cy);
	    if (f < dmin)
	    {
		dmin = f;
		rv = itc.currentKey();
		xval = c->x(i);
		yval = c->y(i);
		index = i;
	    }
	}
	
	++itc;
    }
    dist = int(sqrt(dmin));
    return rv;

}



/*!
  \return the style of the curve indexed by key
  \param key Key of the curve
  \sa setCurveStyle()
*/
QwtCurve::CurveStyle QwtPlot::curveStyle(long key) const
{
    QwtPlotCurve *c;

    if ((c = d_curves->find(key)))
    {
	return c->style();
    }
    else
       return (QwtCurve::CurveStyle) 0;
}

/*!
  \brief the symbol of the curve indexed by key
  \param key Key of the curve
  \return The symbol of the specified curve. If the key is invalid,
          a symbol of type 'NoSymbol'.
*/
const QwtSymbol & QwtPlot::curveSymbol(long key) const
{
    QwtPlotCurve *c;

    if ((c = d_curves->find(key)))
    {
	return c->symbol();
    }
    else
       return DummySymbol();
}

/*!
  \return the pen of the curve indexed by key
  \param key Key of the curve
*/
const QPen& QwtPlot::curvePen(long key) const
{
    QwtPlotCurve *c;

    if ((c = d_curves->find(key)))
    {
	return c->pen();
    }
    else
       return DummyPen();
    
}

/*!
  \return the drawing options of the curve indexed by key
  \param key Key of the curve
*/
int QwtPlot::curveOptions(long key) const
{
    QwtPlotCurve *c;

    if ((c = d_curves->find(key)))
    {
	return c->options();
    }
    else
       return 0;

}

/*!
  \return the spline size of the curve indexed by key
  \param key Key of the curve
*/
int QwtPlot::curveSplineSize(long key) const
{
    QwtPlotCurve *c;

    if ((c = d_curves->find(key)))
    {
	return c->splineSize();
    }
    else
       return 0;
}

/*!
  \return the title of the curve indexed by key
  \param key Key of the curve
*/
QString QwtPlot::curveTitle(long key) const
{
    QwtPlotCurve *c;

    if ((c = d_curves->find(key)))
    {
	return c->title();
    }
    else
       return QString("");
}

/*!
  \return an array containing the keys of all curves
*/
QArray<long> QwtPlot::curveKeys() const
{
    int i;
    QIntDictIterator<QwtPlotCurve> ic(*d_curves);
    QArray<long> rv(d_curves->count());

    ic.toFirst();
    i = 0;
    while (ic.current() && (uint(i) < rv.size()))
    {
	rv[i] = ic.currentKey();
	++ic;
	++i;
    }	
    return rv;
}

/*!
  \brief Return the index of the x axis to which a curve is mapped
  \param key Key of the curve
  \return x axis of the curve or -1 if the key is invalid.
*/
int QwtPlot::curveXAxis(long key) const
{
    QwtPlotCurve *c;

    if ((c = d_curves->find(key)))
    {
	return c->xAxis();
    }
    else
       return -1;    
}


/*!
  \brief the index of the y axis to which a curve is mapped
  \param key Key of the curve
  \return y axis of the curve or -1 if the key is invalid.
*/
int QwtPlot::curveYAxis(long key) const
{
    QwtPlotCurve *c;

    if ((c = d_curves->find(key)))
    {
	return c->yAxis();
    }
    else
       return -1;    
}


/*!
  \brief Generate a unique key for a new curve
  \return new unique key or 0 if no key could be found.
*/
long QwtPlot::newCurveKey()
{
    long newkey = d_curves->count() + 1;

    if (newkey > 1)			// size > 0: check if key exists
    {
	if (d_curves->find(newkey))	// key size+1 exists => there must be a
					// free key <= size
	{
	    // find the first available key <= size
	    newkey = 1;
	    while (newkey <= long(d_curves->count()))
	    {
		if (d_curves->find(newkey))
		   newkey++;
		else
		   break;
	    }

	    // This can't happen. Just paranoia.
	    if (newkey > long(d_curves->count()))
	    {
		while (!d_curves->find(newkey))
		{
		    newkey++;
		    if (newkey > 10000) // prevent infinite loop
		    {
			newkey = 0;
			break;
		    }
		}
	    }
	}
    }
    return newkey;
}

/*!
  \brief Insert a new curve
  \param key unique key
  \param title title of the curve. Default: 0.
  \param xAxis x axis to be attached. Default: xBottom.
  \param yAxis y axis to be attached. Default: yLeft.
  \return \c TRUE if the curve has been successfully inserted,
          \c FALSE if the specified key already exists or if
	  the memory allocation failed.
*/
bool QwtPlot::insertCurve(long key, const char *title, int xAxis, int yAxis)
{
    bool rv = FALSE;
    
    if (d_curves->find(key) == 0)
    {
	QwtPlotCurve *nc = new QwtPlotCurve(this);
	if (nc) 
	{
	    nc->setXAxis(verifyXAxis(xAxis));
	    nc->setYAxis(verifyYAxis(yAxis));
	    nc->setTitle(title);
	    d_curves->insert(key, nc);
	    if (d_legendEnabled) 
	    {
	    	d_legend->appendItem(nc->title(), nc->symbol(),
				    nc->pen(), key);
	    }
	    autoRefresh();
	    rv = TRUE;
	}
    }
    return rv;
}


/*!
  \brief Insert a new curve and return a unique key
  \param title title of the new curve
  \param xAxis x axis to be attached. Defaults to xBottom.
  \param yAxis y axis to be attached. Defaults to yLeft.
  \return The key of the new curve or 0 if no new key could be found
          or if no new curve could be allocated.
*/
long QwtPlot::insertCurve(const char *title, int xAxis, int yAxis)
{

    long newkey = newCurveKey();

    if (newkey > 0)
    {
	insertCurve(newkey, title, xAxis, yAxis);
    }
    return newkey;
}

/*!
  \brief remove the curve indexed by key
  \param key Key of the curve
*/
bool QwtPlot::removeCurve(long key)
{
    if (d_curves->remove(key))
    {
	if (d_legendEnabled)
	{
	    d_legend->removeItem(d_legend->findFirstKey(key));
	}
	
	autoRefresh();
	return TRUE;
    }
    else
       return FALSE;
}

/*!
  \brief Assign a pen to a curve indexed by key
  \param key Key of the curve
  \param pen new pen
  \return \c TRUE if the curve exists
*/
bool QwtPlot::setCurvePen(long key, const QPen &pen)
{
    bool rv = FALSE;
    
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setPen(pen);
	if (d_legendEnabled)
	{
	    d_legend->setPen(d_legend->findFirstKey(key), pen);
	}
	rv = TRUE;
    }
    return rv;
    
}

/*!
  \brief Assign a symbol to a curve indexed by key
  \param key Key of the curve
  \param s
  \return \c TRUE if the curve exists
*/
bool QwtPlot::setCurveSymbol(long key, const QwtSymbol &s)
{
    bool rv = FALSE;
    
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setSymbol(s);
	if (d_legendEnabled)
	{
	    d_legend->setSymbol(d_legend->findFirstKey(key),s);
	}
	rv = TRUE;
    }
    return rv;
}

/*!
  \brief Attach a curve to the specified arrays without copying

  This function does not copy the values of the specified arrays.
  Removing the curve will not delete the data.

  \param key Key of the curve
  \param xdat
  \param ydat Pointers to x and y data
  \param size size of ydat and ydat
  \return \c TRUE if the curve exists
  \warning Don't delete dynamically allocated data while a curve
           is attached to them.
  \sa setCurveData(), QwtCurve::setRawData
*/
bool QwtPlot::setCurveRawData(long key, double *xdat, double *ydat, int size)
{
    bool rv = FALSE;
    
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setRawData(xdat, ydat, size);
	rv = TRUE;
    }
    return rv;
    
}


/*!
  \todo Documentation
*/
bool QwtPlot::setCurveRawData(long key, double *xdat, double *ydat,
			      double *edat, int size)
{
    bool rv = FALSE;

    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
        c->setRawData(xdat, ydat, edat, size);
        rv = TRUE;
    }
    return rv;

}



/*!
  \param key
  \param s
  \todo Documentation
*/
bool QwtPlot::setCurveTitle(long key, const char *s)
{
    bool rv = FALSE;
    
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setTitle(s);
	rv = TRUE;
    }
    return rv;
    
}

/*!
  \brief Assign x and y values to a curve.
  In opposite to @QwtPlot::setCurveRawData@, this function makes
  a 'deep copy' of the data.

  \param key curve key
  \param xdat pointer to x values
  \param ydat pointer to y values
  \param size size of xdat and ydat
  \sa setCurveRawData(), QwtCurve::setData
*/
bool QwtPlot::setCurveData(long key, double *xdat, double *ydat, int size)
{
    bool rv = FALSE;
    
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setData(xdat, ydat, size);
	rv = TRUE;
    }
    return rv;
}
    
bool QwtPlot::setCurveData(long key, double *xdat, double *ydat, 
			   double *edat, int size)
{
   bool rv = FALSE;
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
        c->setData(xdat, ydat, edat, size);
        rv = TRUE;
    }
    return rv;
}

/*!
  \brief Change a curve's style
  \param key Key of the curve
  \param s display style of the curve
  \param options style options
  \return \c TRUE if the curve exists
  \sa QwtCurve::setStyle() for a detailed description of valid styles.
*/
bool QwtPlot::setCurveStyle(long key, QwtCurve::CurveStyle s, int options)
{
    bool rv = FALSE;
    
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setStyle(s, options);
	rv = TRUE;
    }
  
    return rv;
  
}

/*!
  \brief Set the style options of a curve indexed by key
  \param key The curve's key
  \param opt The type of the spline
  \return \c TRUE if the specified curve exists.
  \sa QwtCurve::setOptions for a detailed description of valid options.
*/
bool QwtPlot::setCurveOptions(long key, int opt)
{
    bool rv = FALSE;
    
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setOptions(opt);
	rv = TRUE;
    }
    return rv;
}

/*!
  \brief Set the number of interpolated points of a curve indexed by key
  \param key key of the curve
  \param s size of the spline
  \return \c TRUE if the curve exists
*/
bool QwtPlot::setCurveSplineSize(long key, int s)
{
    bool rv = FALSE;
    
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setSplineSize(s);
	rv = TRUE;
    }
    return rv;
    
}


/*!
  \brief Attach a curve to an x axis
  \param key key of the curve
  \param axis x axis to be attached
*/
bool QwtPlot::setCurveXAxis(long key, int axis)
{
    bool rv = FALSE;

    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setXAxis(verifyXAxis(axis));
	rv = TRUE;
    }

    return rv;
    
    
}

/*!
  \brief Attach a curve to an y axis
  \param key key of the curve
  \param axis y axis to be attached
*/
bool QwtPlot::setCurveYAxis(long key, int axis)
{
    bool rv = FALSE;

    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setYAxis(verifyYAxis(axis));
	rv = TRUE;
    }

    return rv;
    
}


/*!
  \brief Set the baseline for a specified curve

  The baseline is needed for the curve style QwtCurve::Sticks,
  \param key curve key
  \param ref baseline offset from zero
  \sa QwtCurve::setBaseline
*/
bool QwtPlot::setCurveBaseline(long key, double ref)
{
    bool rv = FALSE;
    
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
    {
	c->setBaseline(ref);
	rv = TRUE;
    }
    return rv;
    
}

/*!
  \brief Return the baseline offset for a specified curve
  \param key curve key
  \return Baseline offset of the specified curve,
          or 0 if the curve doesn't exist
  \sa setCurveBaseline()
*/
double QwtPlot::curveBaseline(long key) const
{
    double rv = 0.0;
    QwtPlotCurve *c;
    if ((c = d_curves->find(key)))
	rv = c->baseline();
    return rv;
    
    
}





