/*-*- c++ -*-******************************************************************
 * Qwt Widget Library 
 * Copyright (C) 1997   Josef Wilgen
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

#include "../include/qwt_plot.h"
#include "../include/qwt_plot_pixframe.h"
#include "../include/qwt_math.h"
#include <qrect.h>

void QwtPlotPixFrame::mousePressEvent(QMouseEvent *e)
{
    const QRect &r = contentsRect();
    QMouseEvent m(QEvent::MouseButtonDblClick, e->pos() - r.topLeft(),
                  e->button(), e->state());

    QPainter p;
    
    p.begin(this);
    
    //
    // Delete active outlines
    //
    if (d_outlineActive)
	drawOutline(p);

    d_outlineActive = FALSE;
    
    //
    // store this point as entry point
    //
    d_lastPoint = e->pos();
    d_entryPoint = e->pos();

    //
    // draw new outline
    //
    if (d_outlineEnabled)
    {
	drawOutline(p);
	d_outlineActive = TRUE;
	
    }
    
    p.end();
    
    d_mousePressed = TRUE;

    parentPlot()->plotPressed(&m);
    
}

void QwtPlotPixFrame::mouseReleaseEvent(QMouseEvent *e)
{
    const QRect &r = contentsRect();
    QMouseEvent m(QEvent::MouseButtonDblClick, e->pos() - r.topLeft(),
                  e->button(), e->state());

    QPainter p;
    
    if (d_outlineActive)
    {
	p.begin(this);
	drawOutline(p);
	p.end();
    }
    
    d_outlineActive = FALSE;
    d_mousePressed = FALSE;

    parentPlot()->plotReleased(&m);
    
}

void QwtPlotPixFrame::mouseMoveEvent(QMouseEvent *e)
{
    const QRect &r = contentsRect();
    QMouseEvent m(QEvent::MouseButtonDblClick, e->pos() - r.topLeft(),
                  e->button(), e->state());
    QPainter p;

    if (d_outlineActive)
    {
	p.begin(this);
	drawOutline(p);
	d_lastPoint = e->pos();
	drawOutline(p);
	p.end();
    }
    parentPlot()->plotMoved(&m);
    
}

void QwtPlotPixFrame::paintEvent(QPaintEvent *e)
{
    QRect r = contentsRect();
    QRect ri;
    QPainter p;

    p.begin(this);

    // if necessary, redraw frame
    if (!r.contains(e->rect(), TRUE))
       drawFrame(&p);
    
    // redraw pixmap portion
    if (r.intersects(e->rect()))
    {
	ri = r.intersect(e->rect());

	//
	// erase active outline
	//
	p.save();
	
	if (d_outlineActive)
	   drawOutline(p);

	p.restore();
	
	//
	// repaint pixmap portion
	//
	p.drawPixmap(ri.x(), ri.y(), d_pix,
		     ri.x() - r.x(), ri.y() - r.y(),
		     qwtMin(ri.width(), d_pix.width()),
		     qwtMin(ri.height(), d_pix.height()));

	//
	// redraw outline
	//
	if (d_outlineActive)
	   drawOutline(p);

    }

    p.end();
    
}


void QwtPlotPixFrame::enableOutline(bool tf)
{

    QPainter p;
    p.begin(this);

    //
    //	If the mouse is pressed, erase existing outline
    //  or draw new outline if 'tf' changes the 'enabled' state.
    //
    if ((tf != d_outlineEnabled) && d_mousePressed)
    {
	drawOutline(p);
	d_outlineActive = tf;
    }
    d_outlineEnabled = tf;

    p.end();

}


void QwtPlotPixFrame::setOutlineStyle(Qwt::Shape os)
{

    QPainter p;
    
    p.begin(this);
    
    //
    // erase old outline
    //
    if (d_outlineActive)
	drawOutline(p);

    d_outline = os;

    //
    // draw new outline 
    //
    if (d_outlineActive)
	drawOutline(p);

    p.end();
    
}

void QwtPlotPixFrame::setOutlinePen(const QPen &c)
{
    d_pen = c;
}

void QwtPlotPixFrame::drawOutline(QPainter &p)
{

    
    const QRect &r = this->contentsRect();
    QPen pn = d_pen;
    
    pn.setColor(QColor(0, (pixmapBackground().pixel() ^ d_pen.color().pixel())));
    
    p.setPen(pn);
    p.setRasterOp(XorROP);
    p.setClipRect(this->contentsRect());
    p.setClipping(TRUE);

    switch(d_outline)
    {
    case Qwt::VLine:

	p.drawLine(d_lastPoint.x(), r.top(), d_lastPoint.x(), r.bottom());
	break;
	
    case Qwt::HLine:

	p.drawLine(r.left(), d_lastPoint.y(), r.right(), d_lastPoint.y());
	break;
	
    case Qwt::Cross:

	p.drawLine(r.left(), d_lastPoint.y(), r.right(), d_lastPoint.y());
	p.drawLine(d_lastPoint.x(), r.top(), d_lastPoint.x(), r.bottom());
	break;

    case Qwt::Rect:
	
	p.drawRect(d_entryPoint.x(), d_entryPoint.y(),
		   d_lastPoint.x() - d_entryPoint.x() + 1,
		   d_lastPoint.y() - d_entryPoint.y() + 1);
	break;
	
    case Qwt::Ellipse:

	p.drawEllipse(d_entryPoint.x(), d_entryPoint.y(),
		   d_lastPoint.x() - d_entryPoint.x() + 1,
		   d_lastPoint.y() - d_entryPoint.y() + 1);
	break;
    default:
	break;
    }
}


