/*-*- c++ -*-******************************************************************
 * Qwt Widget Library 
 * Copyright (C) 1997   Josef Wilgen
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

#include "../include/qwt_scale.h"
#include "../include/qwt_math.h"


//------------------------------------------------------------
//.U MEMBER FUNCTION DESCRIPTION
//------------------------------------------------------------

/*!
  \brief Constructor
  \param ori Orientation. Must be one of
             <code>QwtScale::Left, QwtScale::Right, QwtScale::Bottom,
	     </code>or \c QwtScale::Top
  \param parent
  \param name passed to QWidget's CTOR
*/
QwtScale::QwtScale(Orientation ori, QWidget *parent, const char *name)
: QWidget(parent,name)
{
    QwtScaleDraw::Orientation sdo;
    d_borderDist[0] = 0;
    d_borderDist[1] = 0;
    d_baseDist = 4;
    d_titleOffset = 0;
    d_titleDist = 8;

    switch(ori)
    {
    case QwtScale::Left:
	sdo = QwtScaleDraw::Left;
	break;
    case QwtScale::Right:
	sdo = QwtScaleDraw::Right;
	break;
    case QwtScale::Bottom:
	sdo = QwtScaleDraw::Bottom;
	break;
    case QwtScale::Top:
	sdo = QwtScaleDraw::Top;
	break;
    default:
	sdo = QwtScaleDraw::Right;
    }
    
    d_scale.setGeometry(0,0,10,sdo);
    setMinimumSize(30,30);
}

//! Destroy a QwtScale object
QwtScale::~QwtScale()
{
}


/*!
  \brief Assign a title
  \param title new title
*/
void QwtScale::setTitle(const char *title)
{
    d_title = QString(title);
    repaint();
}


/*!
  Specify distances of the scale's endpoints from the
  widget's borders.
  \param dist1 Left or top Distance
  \param dist2 Right or bottom distance
*/
void QwtScale::setBorderDist(int dist1, int dist2)
{
    d_borderDist[0] = dist1;
    d_borderDist[1] = dist2;
    resizeScale();
    repaint();
}


/*!
  \brief Specify the border distance of the scale's base line.
  \param bd border distance of the baseline
*/
void QwtScale::setBaselineDist(int bd)
{
    d_baseDist = qwtMax(0,bd);
    repaint();
}

/*!
  \brief Change the title font
  \param f new font
*/
void QwtScale::setTitleFont(const QFont &f)
{
    d_titleFont = f;
    repaint();
}

/*!
  \brief Change the title color
  \param c new title color
*/
void QwtScale::setTitleColor(const QColor &c)
{
    d_titleColor = c;
    repaint();
}

/*!
  \brief Set the number format for the major scale labels

  Format character and precision have the same meaning as for the
  QString class.
  \param f  -- format character
  \param prec -- precision
  \sa QString::setNum in the Qt manual
*/

void QwtScale::setLabelFormat(char f, int prec)
{
	d_scale.setLabelFormat(f, prec);
}

/*!
  Format character and precision have the same meaning as for the
  QString class.
  \param f format character
  \param prec precision
  \return the number format for the major scale labels
  \sa QString::setNum in the Qt manual
*/

void QwtScale::labelFormat(char &f, int &prec) const
{
	d_scale.labelFormat(f, prec);
}

void QwtScale::paintEvent(QPaintEvent *e)
{
    int th, ta, ttop;
   	 
    QRect r = this->rect();
    
    QPainter p;
    p.begin(this);

    ta = p.fontMetrics().ascent();
    th = p.fontMetrics().height();
    ttop = ta + d_titleOffset;

    // draw scale
    d_scale.draw(&p);

    // draw title
    p.setFont(d_titleFont);
    switch(d_scale.orientation())
    {
    case QwtScaleDraw::Left:
	p.rotate(-90.0);
	ttop = qwtMin(ttop + (r.width() - ttop) / 4, r.width());
	p.drawText(-r.bottom(), r.right() - ttop,
	   r.height(), 2*th,AlignHCenter|AlignTop, d_title);
	break;
    case QwtScaleDraw::Right:
    
	p.rotate(90.0);
	ttop = qwtMin(ttop + (r.width() - ttop) / 4, r.width());
	p.drawText(r.top(), -r.left() - ttop,
	   r.height(), 2*th, AlignHCenter|AlignTop, d_title);
	break;
    case QwtScaleDraw::Top:
	p.drawText(r.x(), r.bottom() - ttop,
		    r.width(), 2*th,
		    AlignHCenter|AlignTop ,d_title);
	break;
    case QwtScaleDraw::Bottom:
	p.drawText(r.x(), r.top() + d_titleOffset,
		    r.width(), 2*th,
		    AlignHCenter|AlignTop,d_title);
	break;
    default:
	break;
    }

    p.end();
    
}

void QwtScale::resizeEvent(QResizeEvent *e)
{
    resizeScale();
}


//! Resize the scale
void QwtScale::resizeScale()
{
    
    QRect r = this->rect();
    QPainter p;

    p.begin(this);

    switch(d_scale.orientation())
    {
    case QwtScaleDraw::Bottom:
	d_scale.setGeometry(r.x() + d_borderDist[0],
			    r.y() + d_baseDist,
			    r.width() - d_borderDist[1] - d_borderDist[0],
			    QwtScaleDraw::Bottom);

	d_titleOffset = d_titleDist + d_baseDist + d_scale.maxHeight(&p);
	break;
	
    case QwtScaleDraw::Top:
	d_scale.setGeometry(r.x() + d_borderDist[0],
			    r.bottom() - d_baseDist,
			    r.width() - d_borderDist[1] - d_borderDist[0],
			    QwtScaleDraw::Top);
	d_titleOffset =  d_titleDist + d_baseDist + d_scale.maxHeight(&p);
	break;
	
    case QwtScaleDraw::Left:
	d_scale.setGeometry(r.right() - d_baseDist,
			    r.y() + d_borderDist[0],
			    r.height() - d_borderDist[0] - d_borderDist[1],
			    QwtScaleDraw::Left);
	d_titleOffset = d_baseDist + d_titleDist + d_scale.maxWidth(&p,FALSE);
	break;
	
    case QwtScaleDraw::Right:
	d_scale.setGeometry(r.x() + d_baseDist,
			    r.y() + d_borderDist[0],
			    r.height() - d_borderDist[0] - d_borderDist[1],
			    QwtScaleDraw::Right);
	d_titleOffset = d_baseDist + d_titleDist + d_scale.maxWidth(&p,FALSE);
	break;
    default:
	break;
    }
    p.end();
    
}

/*!
  \brief Notify a change of the scale

  This virtual function can be overloaded by derived
  classes. The default implementation repaints the widget.
*/
void QwtScale::scaleChange()
{
    repaint();
}



/*!
  \return a size hint
  \warning This function uses a QPainter internally. Don't call
           it while another QPainter is active.
*/
QSize QwtScale::sizeHint() const
{
    int th,ta, tw;
    
    QPainter p;
    p.begin(this);

    p.setFont(d_titleFont);
    QFontMetrics fm = p.fontMetrics();
   
	th = fm.height();
    ta = fm.ascent();
    tw = fm.width(d_title);
    
    QSize rv;
    switch(d_scale.orientation())
    {
    case QwtScaleDraw::Left:
    case QwtScaleDraw::Right:
	rv.setWidth(d_scale.maxWidth(&p,TRUE) + d_baseDist + ta);
	rv.setHeight(d_scale.scaleDiv().majCnt() * 2 * d_scale.maxHeight(&p));
	break;
    case QwtScaleDraw::Top:
    case QwtScaleDraw::Bottom:
	rv.setHeight(d_scale.maxHeight(&p) + d_baseDist + th + d_titleDist);
	rv.setWidth(d_scale.scaleDiv().majCnt()  * d_scale.maxWidth(&p,TRUE));
	break;
    default:
	break;
    }
    p.end();
    
    return rv;
    
}

/*!
  \brief Determine the minimum Border distance

  This member function returns the minimum distance
  of the scale's endpoints from the widget borders which
  is required for the mark labels to fit into the widget.
  \warning
  <ul><li>This function uses a QPainter internally. Don't call it
          while another QPainter is active.
      <li>The minimum border distance depends on the font.</ul>
  \sa setBorderDist()
*/
int QwtScale::minBorderDist() const
{
    int rv = 0;
    
    QPainter p;
    p.begin(this);
    QFontMetrics fm = p.fontMetrics();
    

    switch(d_scale.orientation())
    {
    case QwtScaleDraw::Left:
    case QwtScaleDraw::Right:
	rv = (fm.ascent() + fm.descent() + 1) / 2;
	break;
    case QwtScaleDraw::Top:
    case QwtScaleDraw::Bottom:
	rv = (d_scale.maxLabelWidth(&p,TRUE) + 1) / 2;
	break;
    default:
	break;
    }
    p.end();
    
    return rv;
    
}

/*!
  \brief Notify a change of the font

  This virtual function may be overloaded by derived widgets.
  The default implementation resizes the scale and repaints
  the widget.
  \param oldfont old font
*/
void QwtScale::fontChange(const QFont &oldfont)
{
    resizeScale();
    repaint();
}



/*!
  \brief Assign a scale division

  The scale division determines where to set the tick
  marks.
  \param sd Scale Division
  \sa For more information about scale divisions, see QwtScaleDiv.
*/
void QwtScale::setScale(const QwtScaleDiv &sd)
{
    if (d_scale.scaleDiv() != sd)
    {
    	d_scale.setScale(sd);
    	resize(this->size());
    	repaint();
    }
}









